#ifndef MCU
#define MCU atiny13a
#endif

#ifndef F_CPU
#define F_CPU 4800000UL
#endif

#include "main.h"
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>


//#############################
//Globals
//#############################

/* This line isn't really quite right.*/
//uint8_t saved_position EEMEM;  // EEPROM storage for fader position

btn_state btn1;

/*Some variables for managing the debouncing*/
volatile uint8_t debounced_state;
volatile uint8_t db_state[MAX_CHECKS];
volatile uint8_t idx;

volatile uint16_t tick_count;

//#############################
//Prototypes
//#############################

void led_blink(void);

uint16_t readADC(void);
void motorCoast(void);
void motorMove(int direction);
uint8_t readButton(void);


//#############################
//MAIN
//#############################

int main(int argc, char **argv)
{ 

    /*
    while(1) {
        uint16_t fader_pos = readADC() >> 2; // Scale 10-bit ADC to 8-bit (0-255)
        uint8_t buttonState = readButton();
        
        if (buttonState == 2) { // Long press - Save position
            eeprom_update_byte(&saved_position, fader_pos);
        } 
        else if (buttonState == 1) { // Short press - Move to saved position
            uint8_t target_pos = eeprom_read_byte(&saved_position);
            if (fader_pos < target_pos - 2) motorMove(1); // Move up
            else if (fader_pos > target_pos + 2) motorMove(-1); // Move down
            else motorCoast(); // Stop when reached position
        } 
        else {
            motorCoast(); // Default to coast mode
        }

        led_blink();
         
    }
    */

    return 0;
}


//#############################
//FUNCTIONS
//#############################


void init_prog(void)
{
    /*Set the debounced state to all high*/
    debounced_state = 0xFF;

    init_btn(&btn1, PIN_SW1, PIN_ACTIVE1);

    /*Wait 5ms for pull-up resistors voltage to become stable.*/
    _delay_ms(5);

    /*check if the eeprom has info. */
    while(! eeprom_is_ready()) {} //Blocks until eeprom is ready.

    //Checks against a bit pattern we defined to represent the start of data.
    if(eeprom_read_byte((uint8_t *)ROM_SP_ADR) == START_PATTERN) {
        //Reads the two bytes representing the two states.       
        btn1.is_active = eeprom_read_byte((uint8_t *)ROM_SS1_ADR);
    }
    else {
        //otherwise we write the init values for the start pattern and states.
        eeprom_write_byte((uint8_t *)ROM_SP_ADR, START_PATTERN);
        eeprom_write_byte((uint8_t *)ROM_SS1_ADR, 0x0);
    }

    btn1.is_pressed = ((PINB & (1<<btn1.input_pin))  == 0);

    /*This is to read if the user wants to change the saved states.*/
    /*Manually read the current switch state*/
    /*
    if(btn1.is_pressed){
        btn1.is_active = ! btn1.is_active;
        eeprom_write_byte((uint8_t *)ROM_SS1_ADR, btn1.is_active);
        blink_bypass1();
    }
    if(btn2.is_pressed){
        btn2.is_active = ! btn2.is_active;
        eeprom_write_byte((uint8_t *)ROM_SS2_ADR, btn2.is_active);
        blink_bypass2();
    }   

    if(btn1.is_active){PORTB |= (1<<btn1.output_pin);}
    if(btn2.is_active){PORTB |= (1<<btn2.output_pin);}
    */

    init_timer0(); 
}



/*
 * ############################
 * DEBOUNCING CODE 
 * ############################
 */ 

/*
 * INPUT: The port to check.
 * OUTPUT: None
 * DESCRIPTION: Updates the global debounced state. This function
 * should be called in a ISR a set rate using the hardware timer.
 */
static inline void debounce_switch() {
    uint8_t i, j;
    db_state[idx] = PINB & 0xFF;
    idx+=1;
    j = 0xff;    
    /*Loop through a number of checks*/
    for(i = 0; i < MAX_CHECKS; i++) {
        j = j & db_state[i];
    }
    debounced_state = j;

    if(idx >= MAX_CHECKS) {
        idx = 0;
    }
}



/*Setup the timer0 on the AVR*/
static inline void init_timer0() { 
    /*Zero out the tick_count var*/
    tick_count = 0;

    /*config to normal mode.*/
    TCCR0A = 0x00;  //stop timer
    TCCR0B = 0x00;  //zero timer

    /*set prescaler*/
    //Set to div64
    TCCR0B |= (1<<CS01)|(1<<CS00);

    
    /*Enable global interrupts*/
    sei();

    /*Enabling timer0 interrupt*/
    TCNT0 = 0;
    TIMSK0 |= (1<<TOIE0);
}


//ISR(TIMER0_OVF_vect)

/*The interrupt service routine for the timer0*/
ISR(TIM0_OVF_vect)
{
    /*Disable global interrupts*/
    cli();

    /*Check the state of the switches*/
    debounce_switch();

    /*Update the tick_count*/
    if(btn1.timer_enabled && btn1.pressed_ticks <= UINT8_MAX){
        btn1.pressed_ticks += 1;
    }

    /*Re-Enable global interrupts*/
    sei();
}
